import boto3
import os
from unittest import TestCase

from integration_test.shared.utils.test_env import TicEnvironment
from tic.shared.aws.kms_client import KmsClient
from tic.shared.configuration.configuration import load_config_from_files
from tic.shared.utils import log_util


class TestBase(TestCase):
    test_env_enum = TicEnvironment.local
    host = ""

    @classmethod
    def setUpClass(cls):
        cls.test_env_enum = TicEnvironment.get_enum(
            value=os.environ.get("TEST_ENV", "local")
        )
        cls.logger = log_util.get_stdout_logger(cls.LOGGER_STR)
        cls.rand_lower_bound = 0
        cls.rand_upper_bound = 99999999999
        cls.aws_profile_name = None

        try:
            cls.ci_commit_short_sha = None
            if cls.test_env_enum == TicEnvironment.local:
                cls.base_url = "http://localhost:8000{0}"

            elif cls.test_env_enum == TicEnvironment.dev:
                cls.aws_profile_name = "tic-dev"
                cls.base_url = "https://dev.intel.threatintel.carbonblack.io{0}"

            elif cls.test_env_enum == TicEnvironment.staging:
                cls.base_url = "https://staging.intel.threatintel.carbonblack.io{0}"
                cls.aws_profile_name = "tic-stg"

            elif cls.test_env_enum == TicEnvironment.prod:
                cls.aws_profile_name = "tic-prod"
                raise Exception("Not allowed to run against production")

            elif cls.test_env_enum == TicEnvironment.ci:
                cls.ci_commit_short_sha = os.environ["CI_COMMIT_SHORT_SHA"]
                cls.base_url = f"http://localhost:8080/api/v1/namespaces/cdc/services/https:horizon-{cls.ci_commit_short_sha}:4443/proxy"
                cls.base_url += "{0}"

            else:
                raise Exception("Failed to match test env variable")

            if cls.aws_profile_name:
                os.environ["AWS_PROFILE"] = cls.aws_profile_name

            # TODO: I think config_path is broken due to test nature
            config_path = f"../../ops/config/us-east-1-{cls.test_env_enum}"
            if cls.test_env_enum == TicEnvironment.ci:
                ci_project_dir = os.environ.get("CI_PROJECT_DIR")
                config_path = f"{ci_project_dir}/src/ops/config/us-east-1-ci"
            elif cls.test_env_enum == TicEnvironment.local:
                # Grab CI config -- it shouldn't really matter, since this is overwritten
                # Assumes repo is located in ~/dev/tic
                config_path = os.path.join(
                    os.path.expanduser("~"), "dev/tic/src/ops/config/us-east-1-ci"
                )
            elif cls.test_env_enum == TicEnvironment.dev:
                config_path = os.path.join(
                    os.path.expanduser("~"), "dev/tic/src/ops/config/us-east-1-dev"
                )

            # Configure auth
            cls.base_config = load_config_from_files(
                None,
                root_key="TICBaseConfig",
                config_location=os.path.abspath(config_path),
            )

            # Hack for now
            if cls.test_env_enum == TicEnvironment.local:
                cls.auth_query_params_dict = {}
                cls.auth_query_params = "?"
                return

            cls.kms_client = KmsClient(
                boto3.Session(
                    profile_name=cls.aws_profile_name,
                    region_name=cls.base_config["TICBaseConfig"]["auth"]["aws_region"],
                )
            )

            cls.user_key = cls.base_config["TICBaseConfig"]["auth_credentials"][
                "access_key_id"
            ].strip()
            cls.user_secret = cls.kms_client.decrypt_string(
                cls.base_config["TICBaseConfig"]["auth_credentials"][
                    "encrypted_secret_access_key"
                ]
            ).strip()
            cls.auth_query_params = "?user-key={}&user-secret={}".format(
                cls.user_key, cls.user_secret
            )
            cls.auth_query_params_dict = {
                "user-key": cls.user_key,
                "user-secret": cls.user_secret,
            }

            # these are used to test permissions
            analyst_test_user_key = "0982600f-1917-4419-93fe-7c8ad9a4ba79"
            analyst_test_user_secret = cls.kms_client.decrypt_string(
                cls.base_config["TICBaseConfig"]["auth_credentials"][
                    "encrypted_test_analyst_secret_access_key"
                ]
            ).strip()

            customer_test_user_key = "5eab2308-82f5-42a5-92f7-20b7b016b76c"
            customer_test_user_secret = cls.kms_client.decrypt_string(
                cls.base_config["TICBaseConfig"]["auth_credentials"][
                    "encrypted_test_customer_secret_access_key"
                ]
            ).strip()

            cls.customer_test_user_auth_query_params = (
                "?user-key={0}&user-secret={1}".format(
                    customer_test_user_key, customer_test_user_secret
                )
            )
            cls.customer_test_user_auth_query_params_dict = {
                "user-key": customer_test_user_key,
                "user-secret": customer_test_user_secret,
            }
            cls.analyst_test_user_auth_query_params = (
                "?user-key={0}&user-secret={1}".format(
                    analyst_test_user_key, analyst_test_user_secret
                )
            )
            cls.analyst_test_user_auth_query_params_dict = {
                "user-key": analyst_test_user_key,
                "user-secret": analyst_test_user_secret,
            }
            cls.admin_test_user_auth_query_params = cls.auth_query_params
            cls.admin_test_user_auth_query_params_dict = cls.auth_query_params_dict
        except Exception as e:
            cls.logger.exception("Error loading configuration")
            raise e
