import os

from sqlalchemy.orm import Session

from testlib.test_base import TestBase, TicEnvironment

from integration_test.ubs.utils.local_config import get_ubs_db_settings
from tic.shared.utils.db_util import build_postgresql_connection_string
from tic.shared.ubs.repository.ubs_db import UbsDbRepository
from tic.shared.ubs.repository.ubs_analysis_db import UbsAnalysisDbRepository
from tic.shared.ubs.models.ubs_analysis_models import Main
from tic.shared.ubs.util import hash_to_binary
from tic.shared.ubs.enums.analysis.stage import UbsAnalysisStage
from tic.shared.ubs.enums.analysis.status import UbsAnalysisStatus
from tic.shared.ubs.enums.analysis.pipeline import UbsAnalysisPipeline
from tic.shared.utils.date_util import utc_now


tic_env = TicEnvironment.get_enum(value=os.environ.get("TEST_ENV", "local"))


class TestUbsDbBase(TestBase):
    @classmethod
    def setUpClass(cls):
        super(TestUbsDbBase, cls).setUpClass()
        cls.files_to_clean_up = set()
        cls.aws_region = "us-east-1"
        cls.kms_key_alias = "alias/tic_master"

        # Default settings that assume not local
        db_port = 18018
        db_host = "localhost"
        db_name = cls.base_config["TICBaseConfig"]["ubs_db"]["db"]
        db_username = cls.base_config["TICBaseConfig"]["ubs_db"]["user"]
        db_password = cls.base_config["TICBaseConfig"]["ubs_db"]["password"]
        db_password_encrypted = cls.base_config["TICBaseConfig"]["ubs_db"][
            "pw_encrypted"
        ]
        db_aws_region = cls.base_config["TICBaseConfig"]["AWS"]["aws_region"]
        cls.db_connection_args = {}

        # Additional config, especially for local usage
        if tic_env == TicEnvironment.ci:
            db_port = 8888
        elif tic_env == TicEnvironment.dev:
            db_port = 18019
        elif tic_env == TicEnvironment.staging:
            db_port = 18020
            cls.test_environment = "staging"

        # Local settings, broken up because of additional dynamic config
        if tic_env == TicEnvironment.local:
            db_password_encrypted = False
            cls.db_connection_args = {"sslmode": "disable"}

            # Get config from k8s
            db_name, db_username, db_password = get_ubs_db_settings()

            # temp local override for connecting to localhost postgres server:
            # db_name = "ubs"
            # db_username = "tic"
            # db_password = "admin"
            # db_port = 5432

        cls.connection_string = build_postgresql_connection_string(
            host=db_host,
            port=db_port,
            db=db_name,
            username=db_username,
            password=db_password,
            decrypt=db_password_encrypted,
            aws_region=db_aws_region,
        )
        cls.main_repository = UbsDbRepository(
            None,
            cls.connection_string,
            cls.connection_string,
            connection_args=cls.db_connection_args,
            statement_timeout=0,
        )
        cls.analysis_repository = UbsAnalysisDbRepository(
            None,
            cls.connection_string,
            cls.connection_string,
            connection_args=cls.db_connection_args,
            statement_timeout=0,
        )

    def set_up_analysis_repository(self, sha256: str, writer: Session):
        # create the main object
        self.analysis_repository._insert_main(sha256, writer)

        self.pipeline_name = "REPUTATION"
        self.pipeline_major, self.pipeline_minor = 2, 0
        self.stage_major, self.stage_minor = 1, 0

        self.analysis_repository._insert_stage_topic(
            UbsAnalysisStage.APC_UPLOAD, "apc-upload-arn", True, writer
        )
        self.analysis_repository._insert_stage_topic(
            UbsAnalysisStage.APC_LOOKUP, "apc-lookup-arn", True, writer
        )
        self.analysis_repository._insert_stage_topic(
            UbsAnalysisStage.APC_LOOKUP_POLL, "apc-lookup-arn", True, writer
        )
        self.analysis_repository._insert_stage_topic(
            UbsAnalysisStage.FILE_EXISTS, "N/A", True, writer
        )
        self.analysis_repository._insert_stage_topic(
            UbsAnalysisStage.APC_UPLOAD_ALLOWED, "N/A", True, writer
        )

        # establish mainstagelink
        self.analysis_repository._insert_main_stage_link(
            sha256, UbsAnalysisStage.APC_LOOKUP, writer
        )
        self.analysis_repository._insert_main_stage_link(
            sha256, UbsAnalysisStage.APC_LOOKUP_POLL, writer
        )
        self.analysis_repository._insert_main_stage_link(
            sha256, UbsAnalysisStage.APC_UPLOAD, writer
        )
        self.analysis_repository._insert_main_stage_link(
            sha256, UbsAnalysisStage.APC_UPLOAD_ALLOWED, writer
        )
        self.analysis_repository._insert_main_stage_link(
            sha256, UbsAnalysisStage.FILE_EXISTS, writer
        )

        # establish mainpipelinelink
        self.analysis_repository._insert_main_pipeline_link(
            sha256, self.pipeline_name, writer
        )

        # establish status of pipeline
        self.analysis_repository._insert_pipeline_status(
            sha256,
            self.pipeline_name,
            self.pipeline_major,
            self.pipeline_minor,
            UbsAnalysisStatus.IN_PROGRESS,
            writer,
        )

        self.main_id = (
            writer.query(Main.id).filter(Main.sha256 == hash_to_binary(sha256)).first()
        )

    def set_up_main_repository(self, sha256: str, analysis_writer: Session):
        sha256nobinary = sha256
        magic_str = "PE32+ executable x86-64, for MS Windows"

        # Set up ubap_schema_tables
        self.analysis_repository.update_magic_and_file_details(
            sha256=sha256nobinary,
            sha256_truncated=sha256nobinary,
            s3_key="s3/key/test.zip",
            truncated=False,
            magic_value=magic_str,
            writer=analysis_writer,
        )

        # validate magic and file details were properly set up in ubap schema
        details = self.analysis_repository.get_magic_and_file_details(
            sha256nobinary, analysis_writer
        )
        expectedRes = {
            "sha256_truncated": sha256nobinary,
            "s3_key": "s3/key/test.zip",
            "truncated": False,
            "magic_value": magic_str,
        }
        # assert that the returned dictionary is equal to what was inserted.
        self.assertDictEqual(details[0], expectedRes)

    def complete_reputation_pipeline(self, sha256: str, writer: Session):
        # insert stage status
        self.analysis_repository.update_stages_pipelines_get_targets(
            sha256,
            UbsAnalysisStage.APC_LOOKUP,
            UbsAnalysisStatus.COMPLETE,
            (self.stage_major, self.stage_minor),
            utc_now(),
            writer,
        )
        self.analysis_repository.update_stages_pipelines_get_targets(
            sha256,
            UbsAnalysisStage.APC_UPLOAD_ALLOWED,
            UbsAnalysisStatus.COMPLETE,
            (self.stage_major, self.stage_minor),
            utc_now(),
            writer,
        )
        self.analysis_repository.update_stages_pipelines_get_targets(
            sha256,
            UbsAnalysisStage.FILE_EXISTS,
            UbsAnalysisStatus.COMPLETE,
            (self.stage_major, self.stage_minor),
            utc_now(),
            writer,
        )
        self.analysis_repository.update_stages_pipelines_get_targets(
            sha256,
            UbsAnalysisStage.APC_UPLOAD,
            UbsAnalysisStatus.COMPLETE,
            (self.stage_major, self.stage_minor),
            utc_now(),
            writer,
        )
        self.analysis_repository.update_stages_pipelines_get_targets(
            sha256,
            UbsAnalysisStage.APC_LOOKUP_POLL,
            UbsAnalysisStatus.COMPLETE,
            (self.stage_major, self.stage_minor),
            utc_now(),
            writer,
        )

        pipeline_status = self.analysis_repository.start_pipeline(
            writer, sha256, UbsAnalysisPipeline.REPUTATION
        )
        self.assertEqual(UbsAnalysisStatus.COMPLETE, pipeline_status.status)
