from sqlalchemy.orm import Session
from sqlalchemy.event import listens_for, remove

from testlib.test_ubs_db_base import TestUbsDbBase


def ubs_db_rollback(main_db: bool = False, analysis_db: bool = False):
    def decorator(fn):
        def decorated(self, *args, **kwargs):
            if not isinstance(self, TestUbsDbBase):
                raise TypeError("Test class must be instance of TestUbsDbBase")
            if main_db is analysis_db is False:
                raise AttributeError("You must set at least one DB to rollback")

            if main_db:
                if analysis_db:  # Rollback main and analysis repos
                    rollback_both(
                        self.main_repository.writer_engine,
                        self.analysis_repository.writer_engine,
                        fn,
                        self,
                        *args,
                        **kwargs
                    )
                else:  # Rollback main repo
                    rollback(
                        self.main_repository.writer_engine, fn, self, *args, **kwargs
                    )
            elif analysis_db:  # Rollback analysis repo
                rollback(
                    self.analysis_repository.writer_engine, fn, self, *args, **kwargs
                )

        return decorated

    return decorator


def rollback(engine, fn, self, *args, **kwargs):
    """
    ref: https://stackoverflow.com/a/36509121
    """
    conn = engine.connect()
    trans = conn.begin()
    session = Session(engine)
    session.begin_nested()

    @listens_for(session, "after_transaction_end")
    def restart_savepoint(_session, transaction):
        if transaction.nested and not transaction._parent.nested:
            _session.begin_nested()

    try:
        fn(self, session, *args, **kwargs)
    finally:
        remove(session, "after_transaction_end", restart_savepoint)
        session.rollback()
        session.close()
        trans.rollback()
        trans.close()
        conn.invalidate()


# TODO: This could be more DRY
def rollback_both(main_engine, analysis_engine, fn, self, *args, **kwargs):
    main_conn = main_engine.connect()
    main_trans = main_conn.begin()
    main_session = Session(main_engine)
    main_session.begin_nested()

    @listens_for(main_session, "after_transaction_end")
    def restart_main(_session, transaction):
        if transaction.nested and not transaction._parent.nested:
            _session.begin_nested()

    analysis_conn = analysis_engine.connect()
    analysis_trans = analysis_conn.begin()
    analysis_session = Session(analysis_engine)
    analysis_session.begin_nested()

    @listens_for(analysis_session, "after_transaction_end")
    def restart_analysis(_session, transaction):
        if transaction.nested and not transaction._parent.nested:
            _session.begin_nested()

    try:
        fn(self, main_session, analysis_session, *args, **kwargs)
    finally:
        remove(main_session, "after_transaction_end", restart_main)
        main_session.rollback()
        main_session.close()
        main_trans.rollback()
        main_trans.close()
        main_conn.invalidate()

        remove(analysis_session, "after_transaction_end", restart_analysis)
        analysis_session.rollback()
        analysis_session.close()
        analysis_trans.rollback()
        analysis_trans.close()
        analysis_conn.invalidate()
